/*
 *      $Source: /usr/local/libdata/cvs/simons/petidomo/src/htmlconf/config.c,v $
 *      $Revision: 1.9 $
 *      $Date: 1998/06/01 21:36:41 $
 *
 *      Copyright (C) 1996 by Peter Simons, Germany
 *      All rights reserved.
 */

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>

#include <lists.h>
#include <configfile.h>
#include <petidomo.h>

static struct PD_Config *  MasterConfig;
List ListConfigs;

static uid_t petidomo_uid = 0;

void
SetUIDPetidomo(void)
{
    if (petidomo_uid == 0)	/* no petidomo user */
      return;

    if (getuid() != 0 && geteuid() != 0)
      return;			/* no root privileges */

    setuid(0);
    seteuid(petidomo_uid);
}

void
SetUIDRoot(void)
{
    if (petidomo_uid == 0)	/* no petidomo user */
      return;
    else
      seteuid(0);
}

static char *             fqdn = NULL;
static char *             master_password = NULL;
static char *             debug;
static char *             mta = "/usr/sbin/sendmail";
static char *             mta_options = "-f%s";
static bool               detach = FALSE;
static bool               show_stats = TRUE;

int
InitPetidomo(void)
{
    struct passwd *    pwd;
    char *             basedir = "/usr/local/petidomo";
    int                rc;

    /* Format description of our global config file. */

    struct ConfigFile  MasterCF[] = {
	{ "Hostname", CF_STRING, &fqdn },
	{ "AdminPassword", CF_STRING, &master_password },
	{ "Debug", CF_STRING, &debug },
	{ "MTA", CF_STRING, &mta },
	{ "MTA_Options", CF_STRING, &mta_options },
	{ "DetachImmediately", CF_YES_NO, &detach },
        { "ShowStatistics", CF_YES_NO, &show_stats },
	{ NULL, 0, NULL}
    };

    /* Allocate memory for the global config structure. */

    MasterConfig = (struct PD_Config *) calloc(sizeof(struct PD_Config), 1);
    if (MasterConfig == NULL) {
	syslog(LOG_CRIT, "Failed to allocate %d byte of memory.", sizeof(struct PD_Config));
	return -1;
    }

    /* Init the list of read list configs. */

    ListConfigs = InitList(NULL);

    /* First of all, determine the home directory of the "petidomo"
       user. This will be our base directory for all operations. */

    pwd = getpwnam("petidomo");
    if (pwd != NULL) {
	petidomo_uid = pwd->pw_uid;
	if (strcmp(basedir, pwd->pw_dir) != 0)
	  basedir = xstrdup(pwd->pw_dir); /* Replace the default above. */
	endpwent();
    }
    else
      syslog(LOG_WARNING, "WARNING: User \"petidomo\" not found.");

    /* chdir() into the base directory. */

    rc = chdir(basedir);
    if (rc != 0) {
	syslog(LOG_ERR, "Failed to change current directory to \"%s\": %m", basedir);
	return -1;
    }

    /* Parse the config file. */

    rc = ReadConfig("etc/petidomo.conf", MasterCF);
    if (rc != 0) {
	syslog(LOG_ERR, "Failed to parse the master config file \"petidomo.conf\"");
	return -1;
    }

    /* Copy the results to the structure. */

    MasterConfig->basedir = basedir;
    MasterConfig->fqdn = fqdn;
    MasterConfig->master_password = master_password;
    MasterConfig->mta = mta;
    MasterConfig->mta_options = mta_options;
    MasterConfig->detach = detach;
    MasterConfig->show_stats = show_stats;

    return 0;
}

const struct PD_Config *
getMasterConfig(void)
{
    return MasterConfig;
}

static char *        list_fqdn = NULL;
static char *        admin_password = NULL;
static char *        posting_password = NULL;
static char *        listtype = NULL;
static char *        reply_to = NULL;
static char *        postingfilter = NULL;
static char *        archivepath = NULL;
static bool          allowpubsub = TRUE;
static bool          allowaliensub = TRUE;
static bool          allowmembers = FALSE;
static bool          showonindex = TRUE;

const struct List_Config *
getListConfig(const char * listname)
{
    const struct PD_Config *  MasterConfig;
    struct List_Config *      ListConfig;
    Node                      node;
    int                       rc;
    char                      buffer[4096];

    /* Format description of our global config file. */

    struct ConfigFile ListCF[] = {
	{ "ListType", CF_STRING, &listtype },
	{ "AllowPublicSubscription", CF_YES_NO, &allowpubsub },
	{ "AllowAlienSubscription", CF_YES_NO, &allowaliensub },
	{ "AllowMembersCommand", CF_YES_NO, &allowmembers },
	{ "ShowOnIndex", CF_YES_NO, &showonindex },
	{ "ReplyTo", CF_STRING, &reply_to },
	{ "Hostname", CF_STRING, &list_fqdn },
	{ "AdminPassword", CF_STRING, &admin_password },
	{ "PostingPassword", CF_STRING, &posting_password },
	{ "PostingFilter", CF_STRING, &postingfilter },
	{ "Archive", CF_STRING, &archivepath },
	{ NULL, 0, NULL}
    };

    /* Get the master configuration. */

    MasterConfig = getMasterConfig();

    /* Did we read this config file already? */

    node = FindNodeByKey(ListConfigs, listname);
    if (node != NULL)
      return getNodeData(node);

    /* No? Then read the config file. */

    sprintf(buffer, "lists/%s/config", listname);
    rc = ReadConfig(buffer, ListCF);
    if (rc != 0) {
	syslog(LOG_ERR, "Failed to parse the list \"%s\"'s config file.", listname);
	exit(1);
    }

    /* Set up the list config structure. */

    ListConfig = (struct List_Config *) calloc(sizeof(struct List_Config), 1);
    if (ListConfig == NULL) {
	syslog(LOG_CRIT, "Failed to allocate %d byte of memory.", sizeof(struct List_Config));
	exit(1);
    }
    if (!strcasecmp(listtype, "open"))
      ListConfig->listtype = LIST_OPEN;
    else if (!strcasecmp(listtype, "closed"))
      ListConfig->listtype = LIST_CLOSED;
    else if (!strcasecmp(listtype, "moderated"))
      ListConfig->listtype = LIST_MODERATED;
    else
      ListConfig->listtype = LIST_OPEN;

    ListConfig->allowpubsub = allowpubsub;
    ListConfig->allowaliensub = allowaliensub;
    ListConfig->allowmembers = allowmembers;
    ListConfig->showonindex = showonindex;
    ListConfig->fqdn = (list_fqdn) ? list_fqdn : MasterConfig->fqdn;
    ListConfig->reply_to = reply_to;
    ListConfig->admin_password = admin_password;
    ListConfig->posting_password = posting_password;
    ListConfig->postingfilter = postingfilter;
    ListConfig->archivepath = archivepath;
    AppendNode(ListConfigs, xstrdup(listname), ListConfig);

    return ListConfig;
}
